const CACHE_NAME = "finance";
const URLS = ["/", "/index.html", "/static/js/bundle.js"];
const ACTIONS = {
  ADD_ITEM: "addItem",
};

this.addEventListener("install", (event) => {
  event.waitUntil(
    caches.open(CACHE_NAME).then((cache) => {
      return cache.addAll(URLS);
    })
  );
});

this.addEventListener("fetch", (event) => {
  const url = new URL(event.request.url);
  if (url.protocol === "http:" || url.protocol === "https:") {
    event.respondWith(
      caches.match(event.request).then(async (cachedResponse) => {
        return fetch(event.request)
          .then((response) => {
            if (response.status !== 200) {
              return response;
            }

            const responseCache = response.clone();
            caches.open(CACHE_NAME).then((cache) => {
              cache.put(event.request, responseCache);
            });
            return response;
          })
          .catch(async () => {
            return cachedResponse || new Response("you are offline");
          });
      })
    );
  } else {
    event.respondWith(fetch(event.request));
  }
});

this.addEventListener("message", async (event) => {
  const { action, item, name } = event.data;

  if (action === ACTIONS.ADD_ITEM) {
    const cache = await caches.open(CACHE_NAME);

    const existingCache = await cache.match(name);
    const existingData = await existingCache.json();

    const updatedItem = [...existingData, item];

    const response = new Response(JSON.stringify(updatedItem), {
      headers: { "Content-Type": "application/json" },
    });
    await cache.put(name, response);
  }
});
